
;**************************************************************************
;*                          Stopky                                        *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          Stopky 00:00 - 99:59 (min:sec)                                *
;*          v 1.07 - 05.05.2014                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC12F1840 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   1,2V accu.   *
;**************************************************************************


;Vss (Pin 8) -- GND
;RA0 (Pin 7) -- not used
;RA1 (Pin 6) -- I2C SCL
;RA2 (Pin 5) -- I2C SDA
;RA3 (Pin 4) -- I2C INT
;RA4 (Pin 3) -- Duo LED
;RA5 (Pin 2) -- Duo LED
;Vdd (Pin 1) -- +5V




 INCLUDE "P12F1840.INC"


_U001			     EQU     B'00100110000100'		;see Page 34 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 35 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
TIMER3		EQU	H'22'		;delay routine

v_value		EQU	H'29'
v_MSB		EQU	H'2A'
v_LSB		EQU	H'2B'

v_m1		EQU	H'2C'
v_m2		EQU	H'2D'
v_s1		EQU	H'2E'
v_s2		EQU	H'2F'

v_tmp		EQU	H'30'
v_stop		EQU	H'31'
v_s_char	EQU	H'32'		;special character flag
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;==========================================================================

		ORG	0		;Reset vector address
		goto	start

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

start		nop
;
		clrf	INTCON		;disable interrupts

		movlw	B'00000000'
		movwf	OPTION_REG	;see datasheet Page 151

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11011100'
		movwf	TRISA

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		movlw	B'00111111'
		movwf	WPUA		;enable weak pull-ups

		clrf	BSR		;select BANK 0

		clrf	PORTA
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
		call	init_AMS
;----------------------------------------------------------------------------
start_up	nop

		movlw	B'11111111'		;register XFh --> display test
		movwf	v_MSB
		movlw	B'11111111'		;last bit = 1 --> display test
		movwf	v_LSB
		call	send_data
		call	dr2
		call	dr2
		call	dr2

		movlw	B'11111111'		;register XFh --> display test
		movwf	v_MSB
		movlw	B'11111110'		;last bit = 1 --> mormal operation
		movwf	v_LSB
		call	send_data

		movlw	D'010'
		movwf	v_m1
		movwf	v_m2
		movwf	v_s1
		movwf	v_s2
		call	disp_cnt
		call	dr2
		call	dr2
		call	dr2

re_run		call	init_counter
;--------------------------------------------------------------------
;--------------------------------------------------------------------
main_loop	call	disp_cnt
		call	inc_cnt
		call	d_1sec
		btfss	PORTA,D'004'
		goto	r_reset
		btfsc	v_stop,D'000'
		goto	time_over
		goto	main_loop
;--------------------------------------------------------------------
;--------------------------------------------------------------------
time_over	movlw	D'010'
		movwf	v_m1
		movwf	v_s1
		movlw	D'011'
		movwf	v_m2
		movwf	v_s2
		call	disp_cnt
		call	dr2
		call	dr2
		btfss	PORTA,D'004'
		goto	re_run

		movlw	D'011'
		movwf	v_m1
		movwf	v_s1
		movlw	D'010'
		movwf	v_m2
		movwf	v_s2
		call	disp_cnt
		call	dr2
		call	dr2
		btfss	PORTA,D'004'
		goto	re_run

		goto	time_over
;--------------------------------------------------------------------
;--------------------------------------------------------------------
r_reset		movlw	D'012'
		movwf	v_m1
		movwf	v_m2
		movwf	v_s1
		movwf	v_s2
		call	disp_cnt
		call	dr2

		btfsc	PORTA,D'004'
		goto	re_run

		movlw	D'013'
		movwf	v_m1
		movwf	v_m2
		movwf	v_s1
		movwf	v_s2
		call	disp_cnt
		call	dr2

		btfsc	PORTA,D'004'
		goto	re_run

		movlw	D'014'
		movwf	v_m1
		movwf	v_m2
		movwf	v_s1
		movwf	v_s2
		call	disp_cnt
		call	dr2

		btfsc	PORTA,D'004'
		goto	re_run

		goto	r_reset
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
inc_cnt		nop
		incf	v_s1,1
		movf	v_s1,0
		xorlw	D'010'
		btfss	STATUS,Z
		return
		clrf	v_s1
		incf	v_s2,1
		movf	v_s2,0
		xorlw	D'006'
		btfss	STATUS,Z
		return
		clrf	v_s2
		incf	v_m1,1
		movf	v_m1,0
		xorlw	D'010'
		btfss	STATUS,Z
		return
		clrf	v_m1
		incf	v_m2,1
		movf	v_m2,0
		xorlw	D'010'
		btfss	STATUS,Z
		return
		clrf	v_m2
		bsf	v_stop,D'000'
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
disp_cnt	movlw	H'F1'			;sec LSB
		movwf	v_MSB
		movf	v_s1,0
		call	get_char
		call	send_data

		movlw	H'F2'			;sec MSB
		movwf	v_MSB
		movf	v_s2,0
		call	get_char
		call	send_data

		movlw	H'F3'			;min LSB
		movwf	v_MSB
		movf	v_m1,0
		call	get_char
		btfss	v_s_char,D'000'		;no decimal point for special characters
		bsf	v_LSB,D'007'		;decimal point
		call	send_data

		movlw	H'F4'			;min MSB
		movwf	v_MSB
		movf	v_m2,0
		btfsc	STATUS,Z
		movlw	D'011'			;display space instead of first "0"
		call	get_char
		call	send_data
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
get_char	movwf	v_tmp
		xorlw	D'000'
		btfsc	STATUS,Z
		goto	chr0

		movf	v_tmp,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	chr1

		movf	v_tmp,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	chr2

		movf	v_tmp,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	chr3

		movf	v_tmp,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	chr4

		movf	v_tmp,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	chr5

		movf	v_tmp,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	chr6

		movf	v_tmp,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	chr7

		movf	v_tmp,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	chr8

		movf	v_tmp,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	chr9

		movf	v_tmp,0
		xorlw	D'011'
		btfsc	STATUS,Z
		goto	chr_space

		movf	v_tmp,0
		xorlw	D'012'
		btfsc	STATUS,Z
		goto	chr_s1

		movf	v_tmp,0
		xorlw	D'013'
		btfsc	STATUS,Z
		goto	chr_s2

		movf	v_tmp,0
		xorlw	D'014'
		btfsc	STATUS,Z
		goto	chr_s3

		goto	chrm
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_counter	clrf	v_m1
		clrf	v_m2
		clrf	v_s1
		clrf	v_s2

		clrf	v_stop
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_AMS	movlw	B'11111100'		;register XCh --> Shutdown register
		movwf	v_MSB
		movlw	H'01'			;normal operation
		movwf	v_LSB
		call	send_data
;
		movlw	H'F9'
		movwf	v_MSB			;decode enable register
		movlw	H'00'
		movwf	v_LSB			;no decode mode
		call	send_data
;
		movlw	H'FB'
		movwf	v_MSB			;scan limit register
		movlw	H'03'
		movwf	v_LSB			;all 4 digits
		call	send_data
;
		movlw	H'FA'
		movwf	v_MSB			;intensity register
		movlw	H'0F'
		movwf	v_LSB			;max on
		call	send_data

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
chr0		movlw	B'01111110'
		movwf	v_LSB
		clrf	v_s_char
		return

chr1		movlw	B'00110000'
		movwf	v_LSB
		clrf	v_s_char
		return

chr2		movlw	B'01101101'
		movwf	v_LSB
		clrf	v_s_char
		return

chr3		movlw	B'01111001'
		movwf	v_LSB
		clrf	v_s_char
		return

chr4		movlw	B'00110011'
		movwf	v_LSB
		clrf	v_s_char
		return

chr5		movlw	B'01011011'
		movwf	v_LSB
		clrf	v_s_char
		return

chr6		movlw	B'01011111'
		movwf	v_LSB
		clrf	v_s_char
		return

chr7		movlw	B'01110000'
		movwf	v_LSB
		clrf	v_s_char
		return

chr8		movlw	B'01111111'
		movwf	v_LSB
		clrf	v_s_char
		return

chr9		movlw	B'01111011'
		movwf	v_LSB
		clrf	v_s_char
		return

chrm		movlw	B'00000001'
		movwf	v_LSB
		movlw	H'01'
		movwf	v_s_char
		return

chr_space	movlw	B'00000000'
		movwf	v_LSB
		movlw	H'01'
		movwf	v_s_char
		return

chr_s1		movlw	B'01000000'
		movwf	v_LSB
		movlw	H'01'
		movwf	v_s_char
		return

chr_s2		movlw	B'00000001'
		movwf	v_LSB
		movlw	H'01'
		movwf	v_s_char
		return

chr_s3		movlw	B'00001000'
		movwf	v_LSB
		movlw	H'01'
		movwf	v_s_char
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
send_data	nop

		bcf	PORTA,D'005'		;LOAD = 0

		movf	v_MSB,0
		movwf	v_value
		call	send_byte

		movf	v_LSB,0
		movwf	v_value
		call	send_byte

		bsf	PORTA,D'005'		;LOAD = 1
		nop
		bcf	PORTA,D'005'		;LOAD = 0
		return
;----------------------------------------------------------------------------
send_byte	nop
		bcf	PORTA,D'001'		;CLK = 0
;
		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'007'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'006'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'005'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'004'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'003'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'002'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'001'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'000'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		return
;----------------------------------------------------------------------------
send_bit	bsf	PORTA,D'001'		;CLK = 1
		nop
		bcf	PORTA,D'001'		;CLK = 0
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------










;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------

d2		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1	movlw	D'255'
		movwf	TIMER1

DEL_LOOP2	decfsz	TIMER1,F
		goto	DEL_LOOP2
		decfsz	TIMER2,F
		goto	DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		return
;------------------------------------
dr10		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		return
;------------------------------------
d1		movlw	D'100'
		movwf	TIMER2
DEL_LOOP11	movlw	D'255'
		movwf	TIMER1

DEL_LOOP12	decfsz	TIMER1,F
		goto	DEL_LOOP12
		decfsz	TIMER2,F
		goto	DEL_LOOP11
		return
;------------------------------------

d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;------------------------------------
;------------------------------------
;tuning:
;	005 x 255 x 255 = 325.125 -- a little bit too fast
;	006 x 218 x 250 = 327.000 -- still too fast
;	006 x 219 x 250 = 328.500 -- still too fast
;	006 x 221 x 250 = 331.500 -- very little too slow; but for the application ok...
;------------------------------------
d_1sec		movlw	D'006'
		movwf	TIMER3
DEL_LOOPx0	movlw	D'221'
		movwf	TIMER2
DEL_LOOPx1	movlw	D'250'
		movwf	TIMER1
DEL_LOOPx2	decfsz	TIMER1,F
		goto	DEL_LOOPx2
		decfsz	TIMER2,F
		goto	DEL_LOOPx1
		decfsz	TIMER3,F
		goto	DEL_LOOPx0
		return
;------------------------------------
;------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END